@@ -8,7 +8,7 @@ from copy import copy
 import mimetypes
 from UserDict import DictMixin
 
-from pylons.i18n import ugettext
+from pylons.i18n import ugettext, get_lang
 
 from pylons.configuration import config as pylons_config
 from beaker.middleware import SessionMiddleware, CacheMiddleware
@@ -19,6 +19,7 @@ from paste.deploy.converters import asbool, asint
 
 import tg
 from tg import TGApp
+from tg.configuration.utils import coerce_config
 from tg.util import Bunch, get_partial_dict, DottedFileNameFinder
 
 from routes import Mapper
@@ -153,6 +154,17 @@ class AppConfig(Bunch):
         #override this variable to customize how the tw2 middleware is set up
         self.custom_tw2_config = {}
 
+    def __setitem__(self, key, value):
+        #some entries are required at application setup time
+        #so we need to keep them in sync between tg.config and AppConfig.
+
+        #This is especially true for available renderers which enable
+        #engine check in tg.decorators.Decorator.register_template_engine
+        if key in ['default_renderer', 'renderers']:
+            config[key] = value
+        return super(AppConfig, self).__setattr__(key, value)
+    __setattr__ = __setitem__
+
     def get_root_module(self):
         root_module_path = self.paths['root']
         base_controller_path = self.paths['controllers']
@@ -348,35 +360,26 @@ double check that you have base_config['beaker.session.secret'] = 'mysecretsecre
         # a file system based template lookup mechanism.
         compiled_dir = tg.config.get('templating.mako.compiled_templates_dir', None)
 
-        if not compiled_dir:
-            # Try each given templates path (when are they > 1 ?) for writability..
-            for template_path in self.paths['templates']:
-                if os.access(template_path, os.W_OK):
-                    compiled_dir = template_path
-                    break # first match is as good as any
-
-            # Last recourse: project-dir/data/templates (pylons' default directory)
-            if not compiled_dir:
+        if not compiled_dir or compiled_dir.lower() in ('none', 'false'):
+            # Cache compiled templates in-memory
+            compiled_dir = None
+        else:
+            bad_path = None
+            if os.path.exists(compiled_dir):
+                if not os.access(compiled_dir, os.W_OK):
+                    bad_path = compiled_dir
+                    compiled_dir = None
+            else:
                 try:
-                    root = os.path.dirname(os.path.abspath(self.package.__file__))
-                except AttributeError:
-                    # Thrown during unit tests when self.package.__file__ doesn't exist
-                    root = None
-
-                if root:
-                    pylons_default_path = os.path.join(root, '../data/templates')
-                    if os.access(pylons_default_path, os.W_OK):
-                        compiled_dir = pylons_default_path
-
-                if not compiled_dir:
-                    if use_dotted_templatenames:
-                        # Gracefully digress to in-memory template caching
-                        pass
-                    else:
-                        raise IOError("None of your templates directory, %s, are "
-                            "writable for compiled templates. Please set the "
-                            "templating.mako.compiled_templates_dir variable in your "
-                            ".ini file" % str(self.paths['templates']))
+                    os.makedirs(compiled_dir)
+                except:
+                    bad_path = compiled_dir
+                    compiled_dir = None
+            if bad_path:
+                log.warn("Unable to write cached templates to %r; falling back "
+                         "to an in-memory cache. Please set the `templating.mak"
+                         "o.compiled_templates_dir` configuration option to a "
+                         "writable directory." % bad_path)
 
         if use_dotted_templatenames:
             # Support dotted names by injecting a slightly different template
@@ -479,14 +482,22 @@ double check that you have base_config['beaker.session.secret'] = 'mysecretsecre
         loader = ChoiceLoader(
             [TemplateLoader(path) for path in self.paths['templates']])
 
-        config['pylons.app_globals'].jinja2_env = Environment(loader=loader,
+        config['pylons.app_globals'].jinja2_env = Environment(loader=loader, autoescape=True,
                  auto_reload=self.auto_reload_templates, extensions=self.jinja_extensions)
 
         # Try to load custom filters module under app_package.lib.templatetools
         try:
             filter_package = self.package.__name__ + ".lib.templatetools"
             autoload_lib = __import__(filter_package, {}, {}, ['jinja_filters'])
-            autoload_filters = autoload_lib.jinja_filters.__dict__
+            try:
+                autoload_filters = dict(
+                    map(lambda x: (x,autoload_lib.jinja_filters.__dict__[x]), autoload_lib.jinja_filters.__all__)
+                )
+            except AttributeError:
+                autoload_filters = dict(
+                    filter(lambda x: callable(x[1]),
+                        autoload_lib.jinja_filters.__dict__.iteritems())
+                )
         except (ImportError, AttributeError):
             autoload_filters = {}
 
@@ -538,9 +549,35 @@ double check that you have base_config['beaker.session.secret'] = 'mysecretsecre
 
     def setup_ming(self):
         """Setup MongoDB database engine using Ming"""
-        from ming.datastore import DataStore
+        try:
+            from ming import create_datastore
+            def create_ming_datastore(url, database, **kw):
+                if database and url[-1] != '/':
+                    url += '/'
+                ming_url = url + database
+                return create_datastore(ming_url, **kw)
+        except ImportError:
+            from ming.datastore import DataStore
+            def create_ming_datastore(url, database, **kw):
+                return DataStore(url, database=database, **kw)
 
-        datastore = DataStore(config['ming.url'], database=config['ming.db'])
+        def mongo_read_pref(value):
+            from pymongo.read_preferences import ReadPreference
+            return getattr(ReadPreference, value)
+
+        datastore_options = coerce_config(config, 'ming.connection.', {'max_pool_size':asint,
+                                                                       'network_timeout':asint,
+                                                                       'tz_aware':asbool,
+                                                                       'safe':asbool,
+                                                                       'journal':asbool,
+                                                                       'wtimeout':asint,
+                                                                       'fsync':asbool,
+                                                                       'ssl':asbool,
+                                                                       'read_preference':mongo_read_pref})
+        datastore_options.pop('host', None)
+        datastore_options.pop('port', None)
+
+        datastore = create_ming_datastore(config['ming.url'], config.get('ming.db', ''), **datastore_options)
         config['pylons.app_globals'].ming_datastore = datastore
         self.package.model.init_model(datastore)
 
@@ -708,16 +745,29 @@ double check that you have base_config['beaker.session.secret'] = 'mysecretsecre
                 from tgming import setup_ming_auth
                 app = setup_ming_auth(app, skip_authentication=skip_authentication, **auth_args)
         else:
-            from tg.configuration.auth import setup_auth
-            if 'authenticators' not in auth_args:
+            try:
+                pos = auth_args['authenticators'].index(('default', None))
+            except KeyError:
+                pos = None
+            except ValueError:
+                pos = -1
+            if pos is None or pos >= 0:
                 if self.auth_backend == "sqlalchemy":
                     from tg.configuration.sqla.auth import create_default_authenticator
                     auth_args, sqlauth = create_default_authenticator(**auth_args)
-                    auth_args['authenticators'] = [('sqlauth', sqlauth)]
+                    authenticator = ('sqlauth', sqlauth)
                 elif self.auth_backend == "ming":
                     from tg.configuration.mongo.auth import create_default_authenticator
                     auth_args, mingauth = create_default_authenticator(**auth_args)
-                    auth_args['authenticators'] = [('mingauth', mingauth)]
+                    authenticator = ('mingauth', mingauth)
+                else:
+                    authenticator = None
+                if authenticator:
+                    if pos is None:
+                        auth_args['authenticators'] = [authenticator]
+                    else:
+                        auth_args['authenticators'][pos] = authenticator
+            from tg.configuration.auth import setup_auth
             app = setup_auth(app, skip_authentication=skip_authentication, **auth_args)
 
         return app
@@ -830,6 +880,7 @@ double check that you have base_config['beaker.session.secret'] = 'mysecretsecre
         default_tw2_config = dict( default_engine=self.default_renderer,
                                    preferred_rendering_engines=tw2_engines,
                                    translator=ugettext,
+                                   get_lang=get_lang,
                                    auto_reload_templates=self.auto_reload_templates,
                                    controller_prefix='/tw2/controllers/',
                                    res_prefix='/tw2/resources/',
