@@ -26,6 +26,8 @@ from tg.flash import flash
 
 from tg.predicates import NotAuthorizedError
 
+from pylons.decorators.cache import beaker_cache
+
 import logging
 log = logging.getLogger(__name__)
 
@@ -103,7 +105,11 @@ class Decoration(object):
         default_renderer = config.get('default_renderer')
         available_renderers = config.get('renderers', [])
 
-        if engine and engine not in available_renderers:
+        if engine and not available_renderers:
+            log.warn('Renderers not registered yet while exposing template %s for engine %s, '
+                     'skipping engine availability check', template, engine)
+
+        if engine and available_renderers and engine not in available_renderers:
             log.debug('Registering template %s for engine %s not available. Skipping it', template, engine)
             return
 
@@ -155,8 +161,8 @@ class Decoration(object):
 
         """
 
-        if hasattr(request, 'response_type'
-                ) and request.response_type in self.engines:
+        response_type = getattr(request, 'response_type', None)
+        if response_type is not None and response_type in self.engines:
             accept_types = request.response_type
         else:
             accept_types = request.headers.get('accept', '*/*')
@@ -694,7 +700,7 @@ def with_trailing_slash(remainder, params):
 class _BaseProtectionDecorator(object):
     default_denial_handler = None
 
-    def __init__(self, predicate, denial_handler=None):
+    def __init__(self, predicate, denial_handler=None, smart_denial=False):
         """Verify that the predicate is met.
 
         :param predicate: An object with a check_authorization(environ) method which
@@ -709,6 +715,7 @@ class _BaseProtectionDecorator(object):
 
         self.predicate = predicate
         self.denial_handler = denial_handler or self.default_denial_handler
+        self.smart_denial = smart_denial
 
 
 class require(_BaseProtectionDecorator):
@@ -746,12 +753,12 @@ class require(_BaseProtectionDecorator):
 
     def default_denial_handler(self, reason):
         """Authorization denial handler for protectors."""
-        if response.status_int == 401:
-            status = 'warning'
+        status = 'warning' if response.status_int == 401 else 'error'
+        if not self.smart_denial:
+            flash(reason, status=status)
         else:
-            # Status is a 403
-            status = 'error'
-        flash(reason, status=status)
+            if response.content_type not in ['application/json', 'text/xml']:
+                flash(reason, status=status)
         abort(response.status_int, reason)
 
 
