@@ -1,5 +1,5 @@
 """TurboGears project related information"""
-version = "2.2.1"
+version = "2.2.2"
 description = "Next generation TurboGears built on Pylons"
 long_description="""
 TurboGears brings together a best of breed python tools
@@ -24,8 +24,8 @@ The latest development version is available in the
     https://sourceforge.net/p/turbogears2/tg2/
 """
 url="http://www.turbogears.org/"
-author= "Mark Ramm, Christopher Perkins, Jonathan LaCour, Rick Copland, Alberto Valverde, and the TurboGears community"
-email = "mark.ramm@gmail.com, alberto@toscat.net, m.pedersen@icelus.org"
+author= "Mark Ramm, Christopher Perkins, Jonathan LaCour, Rick Copland, Alberto Valverde, Michael Pedersen, Alessandro Molina, and the TurboGears community"
+email = "mark.ramm@gmail.com, alberto@toscat.net, m.pedersen@icelus.org, amol@turbogears.org"
 copyright = """Copyright 2005-2011 Kevin Dangoor,
 Alberto Valverde, Mark Ramm, Christopher Perkins and contributors"""
 license = "MIT"
